#include "stdafx.h"
#include "DtvEngine.h"
#include "HDUSTest.h"
#include "ChannelManager.h"




CChannelManager::CChannelManager()
{
	m_CurrentSpace=SPACE_INVALID;
	m_CurrentChannel=-1;
	m_ChangingChannel=-1;
	m_fNetworkRemocon=false;
	m_pNetworkRemoconChannelList=NULL;
}


CChannelManager::~CChannelManager()
{
}


static void SkipSpace(char **ppText)
{
	char *p=*ppText;

	while (*p==' ' || *p=='\t')
		p++;
	*ppText=p;
}


bool CChannelManager::LoadChannelList(LPCTSTR pszFileName,CBonSrcDecoder *pSrcDecoder)
{
	HANDLE hFile;
	DWORD Size,Read;
	char *pszText,*p,*q;
	TCHAR szName[MAX_CHANNEL_NAME];
	int NumSpaces;
	int Space,ChannelNo,Channel,ChannelIndex;

	hFile=CreateFile(pszFileName,GENERIC_READ,FILE_SHARE_READ,NULL,
														OPEN_EXISTING,0,NULL);
	if (hFile==INVALID_HANDLE_VALUE)
		return false;
	Size=GetFileSize(hFile,NULL);
	if (Size==0) {
		CloseHandle(hFile);
		return false;
	}
	pszText=new char[Size+1];
	if (!ReadFile(hFile,pszText,Size,&Read,NULL) || Read!=Size) {
		delete [] pszText;
		CloseHandle(hFile);
		return false;
	}
	CloseHandle(hFile);
	m_AllChannelList.Clear();
	if (pSrcDecoder!=NULL)
		NumSpaces=pSrcDecoder->NumSpaces();
	else
		NumSpaces=0;
	pszText[Size]='\0';
	p=pszText;
	do {
		while (*p=='\r' || *p=='\n' || *p==' ' || *p=='\t')
			p++;
		if (*p==';') {	// Rg
			while (*p!='\r' && *p!='\n' && *p!='\0')
				p++;
			continue;
		}
		if (*p=='\0')
			break;
		// `l
		q=p;
		while (*p!=',' && *p!='\0') {
			if (IsDBCSLeadByteEx(CP_ACP,*p) && *(p+1)!='\0')
				p++;
			p++;
		}
		if (*p!=',')
			break;
		*p++='\0';
#ifdef UNICODE
		MultiByteToWideChar(CP_ACP,0,q,-1,szName,MAX_CHANNEL_NAME);
#else
		lstrcpyn(szName,q,MAX_CHANNEL_NAME);
#endif
		SkipSpace(&p);
		// Rԍ
		ChannelNo=0;
		while (*p>='0' && *p<='9') {
			ChannelNo=ChannelNo*10+(*p-'0');
			p++;
		}
		SkipSpace(&p);
		if (*p!=',')
			break;
		p++;
		SkipSpace(&p);
		// `l
		bool fCompatible=false;
		bool fOK=false;
		int Length;
		TCHAR szChannel[64];
		int i,j;
		if (*p=='C' || (*p>='0' && *p<='9')) {
			// `lԍɂw肩(cap_hdusƂ̌݊p)
			Length=0;
			if (*p=='C') {
				Space=1;
				Length++;
			} else
				Space=0;
			Channel=0;
			while (p[Length]>='0' && p[Length]<='9') {
				Channel=Channel*10+(p[Length]-'0');
				Length++;
			}
			if ((p[Length]=='\r' || p[Length]=='\n' || p[Length]=='\0')
					&& Channel>=FIRST_UHF_CHANNEL) {
				wsprintf(szChannel,TEXT("%s%dch"),
										*p=='C'?TEXT("C"):TEXT(""),Channel);
				p+=Length;
				Length=lstrlen(szChannel);
				fCompatible=true;
			}
		}
		if (!fCompatible) {
			q=p;
			while (*p!='\0' && *p!='\r' && *p!='\n')
				p++;
			Length=MultiByteToWideChar(CP_ACP,0,q,p-q,szChannel,lengthof(szChannel));
			Channel=0;
		}
		if (NumSpaces>0) {
			for (i=0;i<NumSpaces;i++) {
				LPCTSTR pszChannel;

				for (j=0;(pszChannel=pSrcDecoder->GetChannelName(i,j))!=NULL;j++) {
					if (_tcsnicmp(pszChannel,szChannel,Length)==0
							&& (pszChannel[Length]==' ' || pszChannel[Length]=='\0')) {
						Space=i;
						ChannelIndex=j;
						fOK=true;
						break;
					}
				}
			}
		} else if (fCompatible) {
			ChannelIndex=Channel-FIRST_UHF_CHANNEL;
			fOK=true;
		}
		if (ChannelNo>0 && fOK)
			m_AllChannelList.AddChannel(Space,Channel,ChannelIndex,ChannelNo,szName);
		while (*p!='\r' && *p!='\n' && *p!='\0')
			p++;
	} while (*p!='\0');
	return true;
}


bool CChannelManager::MakeTuningSpaceList(int NumSpaces)
{
	return m_TuningSpaceList.Create(&m_AllChannelList,NumSpaces);
}


bool CChannelManager::ReserveTuningSpaceList(int NumSpaces)
{
	return m_TuningSpaceList.Reserve(NumSpaces);
}


bool CChannelManager::SetCurrentSpace(int Space)
{
	if (!m_fNetworkRemocon) {
		if (Space!=SPACE_ALL) {
			if (Space<0 || Space>=m_TuningSpaceList.NumSpaces())
				return false;
		}
	}
	m_CurrentSpace=Space;
	m_CurrentChannel=-1;
	return true;
}


bool CChannelManager::SetCurrentChannel(int Channel)
{
	const CChannelList *pList=GetCurrentChannelList();

	if (pList==NULL || Channel<-1)
		return false;
	if (Channel>=pList->NumChannels())
		return false;
	m_CurrentChannel=Channel;
	return true;
}


bool CChannelManager::SetChangingChannel(int Channel)
{
	m_ChangingChannel=Channel;
	return true;
}


bool CChannelManager::SetNetworkRemoconMode(bool fNetworkRemocon,const CChannelList *pList)
{
	if (fNetworkRemocon && pList==NULL)
		return false;
	m_fNetworkRemocon=fNetworkRemocon;
	m_pNetworkRemoconChannelList=pList;
	m_CurrentSpace=0;
	m_CurrentChannel=-1;
	return true;
}


const CChannelInfo *CChannelManager::GetCurrentChannelInfo() const
{
	const CChannelList *pList=GetCurrentChannelList();

	if (pList!=NULL)
		return pList->GetChannelInfo(m_CurrentChannel);
	return NULL;
}


const CChannelInfo *CChannelManager::GetChangingChannelInfo() const
{
	const CChannelList *pList=GetCurrentChannelList();

	if (pList!=NULL)
		return pList->GetChannelInfo(m_ChangingChannel);
	return NULL;
}


const CChannelInfo *CChannelManager::GetNextChannelInfo(bool fNext) const
{
	int Channel,No;
	const CChannelInfo *pInfo;

	if (m_ChangingChannel>=0) {
		Channel=m_ChangingChannel;
	} else {
		if (m_CurrentChannel<0)
			return NULL;
		Channel=m_CurrentChannel;
	}
	if (m_fNetworkRemocon) {
		No=m_pNetworkRemoconChannelList->GetChannelNo(Channel);
		if (fNext)
			No=m_pNetworkRemoconChannelList->GetNextChannelNo(No,true);
		else
			No=m_pNetworkRemoconChannelList->GetPrevChannelNo(No,true);
		pInfo=m_pNetworkRemoconChannelList->GetChannelInfo(
							m_pNetworkRemoconChannelList->FindChannelNo(No));
	} else {
		const CChannelList *pList;

		if (m_CurrentSpace==SPACE_ALL) {
			pList=&m_AllChannelList;
		} else {
			pList=m_TuningSpaceList.GetChannelList(m_CurrentSpace);
			if (pList==NULL)
				return NULL;
		}
		No=pList->GetChannelNo(m_CurrentChannel);
		if (fNext)
			No=pList->GetNextChannelNo(No,true);
		else
			No=pList->GetPrevChannelNo(No,true);
		pInfo=pList->GetChannelInfo(pList->FindChannelNo(No));
	}
	return pInfo;
}


const CChannelList *CChannelManager::GetCurrentChannelList() const
{
	if (m_fNetworkRemocon)
		return m_pNetworkRemoconChannelList;
	if (m_CurrentSpace==SPACE_ALL)
		return &m_AllChannelList;
	return m_TuningSpaceList.GetChannelList(m_CurrentSpace);
}


CChannelList *CChannelManager::GetChannelList(int Space)
{
	//if (m_fNetworkRemocon)
	//	return const_cast<CChannelList*>(m_pNetworkRemoconChannelList);
	if (Space==SPACE_ALL)
		return &m_AllChannelList;
	return m_TuningSpaceList.GetChannelList(Space);
}


const CChannelList *CChannelManager::GetAllChannelList() const
{
	return &m_AllChannelList;
}


int CChannelManager::FindChannelInfo(const CChannelInfo *pInfo) const
{
	const CChannelList *pList=GetCurrentChannelList();
	int i;

	if (pList==NULL)
		return -1;
	for (i=0;i<pList->NumChannels();i++) {
		if (pList->GetChannelInfo(i)==pInfo)
			return i;
	}
	return -1;
}
