#include "stdafx.h"
#include "HDUSTest.h"
#include "StatusView.h"


#define STATUS_WINDOW_CLASS	TEXT("HDUSTest Status")

#define STATUS_MARGIN	4




CStatusItem::CStatusItem(int ID,int DefaultWidth)
{
	m_ID=ID;
	m_DefaultWidth=DefaultWidth;
	m_Width=DefaultWidth;
	m_MinWidth=8;
}


void CStatusItem::SetStatusView(CStatusView *pStatus)
{
	m_pStatus=pStatus;
}


int CStatusItem::GetIndex() const
{
	int i;

	for (i=0;i<m_pStatus->NumItems();i++) {
		if (m_pStatus->GetItem(i)==this)
			return i;
	}
	return -1;
}


bool CStatusItem::GetRect(RECT *pRect) const
{
	return m_pStatus->GetItemRect(m_ID,pRect);
}


bool CStatusItem::SetWidth(int Width)
{
	if (Width<m_MinWidth)
		Width=m_MinWidth;
	else
		m_Width=Width;
	return true;
}


void CStatusItem::SetVisible(bool fVisible)
{
	m_fVisible=fVisible;
	OnVisibleChange(fVisible && m_pStatus->GetVisible());
}




HINSTANCE CStatusView::m_hinst=NULL;


bool CStatusView::Initialize(HINSTANCE hinst)
{
	WNDCLASS wc;

	m_hinst=hinst;
	wc.style=0;
	wc.lpfnWndProc=WndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hInstance=hinst;
	wc.hIcon=NULL;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=NULL;
	wc.lpszMenuName=NULL;
	wc.lpszClassName=STATUS_WINDOW_CLASS;
	return RegisterClass(&wc)!=0;
}


CStatusView::CStatusView()
{
	LOGFONT lf;
	GetObject(GetStockObject(DEFAULT_GUI_FONT),sizeof(LOGFONT),&lf);
	m_hfontStatus=CreateFontIndirect(&lf);
	m_crBackColor=RGB(128,192,160);
	m_crTextColor=RGB(64,96,80);
	m_crHighlightBackColor=RGB(64,96,80);
	m_crHighlightTextColor=RGB(128,192,160);
	m_NumItems=0;
	m_fSingleMode=false;
	m_pszSingleText=NULL;
	m_HotItem=-1;
	m_fTrackMouseEvent=false;
}


CStatusView::~CStatusView()
{
	Destroy();
	for (int i=0;i<m_NumItems;i++)
		delete m_pItemList[i];
	delete [] m_pszSingleText;
}


bool CStatusView::Create(HWND hwndParent,DWORD Style,DWORD ExStyle,int ID)
{
	return CreateBasicWindow(hwndParent,Style,ExStyle,ID,
							 STATUS_WINDOW_CLASS,NULL,m_hinst);
}


const CStatusItem *CStatusView::GetItem(int Index) const
{
	if (Index<0 || Index>=m_NumItems)
		return NULL;
	return m_pItemList[Index];
}


CStatusItem *CStatusView::GetItem(int Index)
{
	if (Index<0 || Index>=m_NumItems)
		return NULL;
	return m_pItemList[Index];
}


const CStatusItem *CStatusView::GetItemByID(int ID) const
{
	int Index=IDToIndex(ID);

	if (Index<0)
		return NULL;
	return m_pItemList[Index];
}


CStatusItem *CStatusView::GetItemByID(int ID)
{
	int Index=IDToIndex(ID);

	if (Index<0)
		return NULL;
	return m_pItemList[Index];
}


bool CStatusView::AddItem(CStatusItem *pItem)
{
	if (m_NumItems>=MAX_STATUS_ITEMS)
		return false;
	m_pItemList[m_NumItems++]=pItem;
	pItem->SetStatusView(this);
	return true;
}


int CStatusView::IDToIndex(int ID) const
{
	int i;

	for (i=0;i<m_NumItems;i++) {
		if (m_pItemList[i]->GetID()==ID)
			return i;
	}
	return -1;
}


int CStatusView::IndexToID(int Index) const
{
	if (Index<0 || Index>=m_NumItems)
		return -1;
	return m_pItemList[Index]->GetID();
}


CStatusView *CStatusView::GetStatusView(HWND hwnd)
{
	return reinterpret_cast<CStatusView*>(GetWindowLongPtr(hwnd,GWLP_USERDATA));
}


LRESULT CALLBACK CStatusView::WndProc(HWND hwnd,UINT uMsg,WPARAM wParam,
																LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			LPCREATESTRUCT pcs=(LPCREATESTRUCT)lParam;
			CStatusView *pStatus;
			LOGFONT lf;
			HDC hdc;
			HFONT hfontOld;
			TEXTMETRIC tm;
			RECT rc;

			pStatus=static_cast<CStatusView*>(pcs->lpCreateParams);
			SetWindowLongPtr(hwnd,GWLP_USERDATA,(LONG_PTR)pStatus);
			hdc=GetDC(hwnd);
			hfontOld=(HFONT)SelectObject(hdc,pStatus->m_hfontStatus);
			GetTextMetrics(hdc,&tm);
			SelectObject(hdc,hfontOld);
			ReleaseDC(hwnd,hdc);
			rc.left=0;
			rc.top=0;
			rc.right=0;
			rc.bottom=tm.tmHeight+STATUS_MARGIN*2;
			AdjustWindowRectEx(&rc,pcs->style,FALSE,pcs->dwExStyle);
			MoveWindow(hwnd,0,0,0,rc.bottom-rc.top,FALSE);
			pStatus->m_HotItem=-1;
			pStatus->m_fTrackMouseEvent=false;
		}
		return 0;

	case WM_PAINT:
		{
			CStatusView *pStatus=GetStatusView(hwnd);
			PAINTSTRUCT ps;
			HBRUSH hbrBk,hbrHighlight;
			HFONT hfontOld;
			COLORREF crOldTextColor,crOldBkColor;
			int OldBkMode;
			RECT rc;
			TCHAR szText[64];
			int i;
			int x;

			BeginPaint(hwnd,&ps);
			hfontOld=(HFONT)SelectObject(ps.hdc,pStatus->m_hfontStatus);
			OldBkMode=SetBkMode(ps.hdc,TRANSPARENT);
			hbrBk=CreateSolidBrush(pStatus->m_crBackColor);
			if (pStatus->m_HotItem>=0)
				hbrHighlight=CreateSolidBrush(pStatus->m_crHighlightBackColor);
			else
				hbrHighlight=NULL;
			crOldTextColor=GetTextColor(ps.hdc);
			crOldBkColor=GetBkColor(ps.hdc);
			pStatus->GetClientRect(&rc);
			if (pStatus->m_fSingleMode) {
				FillRect(ps.hdc,&ps.rcPaint,hbrBk);
				SetTextColor(ps.hdc,pStatus->m_crTextColor);
				rc.left+=STATUS_MARGIN;
				DrawText(ps.hdc,pStatus->m_pszSingleText,-1,&rc,
						DT_LEFT | DT_SINGLELINE | DT_VCENTER | DT_NOPREFIX);
			} else {
				for (i=0;i<pStatus->m_NumItems;i++) {
					if (!pStatus->m_pItemList[i]->GetVisible())
						continue;
					SetTextColor(ps.hdc,i==pStatus->m_HotItem?
						pStatus->m_crHighlightTextColor:pStatus->m_crTextColor);
					SetBkColor(ps.hdc,i==pStatus->m_HotItem?
						pStatus->m_crHighlightBackColor:pStatus->m_crBackColor);
					rc.right=rc.left+pStatus->m_pItemList[i]->GetWidth()+
															STATUS_MARGIN*2;
					FillRect(ps.hdc,&rc,i==pStatus->m_HotItem?hbrHighlight:hbrBk);
					rc.left+=STATUS_MARGIN;
					rc.right-=STATUS_MARGIN;
					pStatus->m_pItemList[i]->Draw(ps.hdc,&rc);
					rc.left=rc.right+STATUS_MARGIN;
				}
				rc.right=ps.rcPaint.right;
				if (rc.right>rc.left)
					FillRect(ps.hdc,&rc,hbrBk);
			}
			DeleteObject(hbrBk);
			if (hbrHighlight!=NULL)
				DeleteObject(hbrHighlight);
			SetBkColor(ps.hdc,crOldBkColor);
			SetTextColor(ps.hdc,crOldTextColor);
			SetBkMode(ps.hdc,OldBkMode);
			SelectObject(ps.hdc,hfontOld);
			EndPaint(hwnd,&ps);
		}
		return 0;

	case WM_MOUSEMOVE:
		if (GetCapture()==hwnd) {
			CStatusView *pStatus=GetStatusView(hwnd);
			int x=GET_X_LPARAM(lParam),y=GET_Y_LPARAM(lParam);
			RECT rc;

			pStatus->GetItemRect(pStatus->m_HotItem,&rc);
			x-=rc.left;
			pStatus->m_pItemList[pStatus->m_HotItem]->OnMouseMove(x,y);
		} else {
			CStatusView *pStatus=GetStatusView(hwnd);
			int x=GET_X_LPARAM(lParam);
			int Left,Right;
			int i;

			Left=0;
			for (i=0;i<pStatus->m_NumItems;i++) {
				if (!pStatus->m_pItemList[i]->GetVisible())
					continue;
				Right=Left+STATUS_MARGIN*2+pStatus->m_pItemList[i]->GetWidth();
				if (x>=Left && x<Right)
					break;
				Left=Right;
			}
			if (i==pStatus->m_NumItems)
				i=-1;
			if (i!=pStatus->m_HotItem) {
				int OldHotItem;

				OldHotItem=pStatus->m_HotItem;
				pStatus->m_HotItem=i;
				if (OldHotItem>=0)
					pStatus->UpdateItem(pStatus->IndexToID(OldHotItem));
				if (pStatus->m_HotItem>=0)
					pStatus->UpdateItem(pStatus->IndexToID(pStatus->m_HotItem));
			}
			if (!pStatus->m_fTrackMouseEvent) {
				TRACKMOUSEEVENT tme;

				tme.cbSize=sizeof(TRACKMOUSEEVENT);
				tme.dwFlags=TME_LEAVE;
				tme.hwndTrack=hwnd;
				if (TrackMouseEvent(&tme))
					pStatus->m_fTrackMouseEvent=true;
			}
		}
		return 0;

	case WM_MOUSELEAVE:
		{
			CStatusView *pStatus=GetStatusView(hwnd);

			if (pStatus->m_HotItem>=0) {
				int i=pStatus->m_HotItem;

				pStatus->m_HotItem=-1;
				pStatus->UpdateItem(pStatus->IndexToID(i));
			}
			pStatus->m_fTrackMouseEvent=false;
		}
		return 0;

	case WM_LBUTTONDOWN:
	case WM_RBUTTONDOWN:
		{
			CStatusView *pStatus=GetStatusView(hwnd);

			if (pStatus->m_HotItem>=0) {
				int x=GET_X_LPARAM(lParam),y=GET_Y_LPARAM(lParam);
				RECT rc;

				pStatus->GetItemRect(pStatus->m_HotItem,&rc);
				x-=rc.left;
				if (uMsg==WM_LBUTTONDOWN)
					pStatus->m_pItemList[pStatus->m_HotItem]->OnLButtonDown(x,y);
				else
					pStatus->m_pItemList[pStatus->m_HotItem]->OnRButtonDown(x,y);
			}
		}
		return 0;

	case WM_LBUTTONUP:
		if (GetCapture()==hwnd) {
			ReleaseCapture();
		}
		return 0;

	case WM_SETCURSOR:
		if (LOWORD(lParam)==HTCLIENT) {
			CStatusView *pStatus=GetStatusView(hwnd);

			if (pStatus->m_HotItem>=0) {
				SetCursor(LoadCursor(NULL,IDC_HAND));
				return TRUE;
			}
		}
		break;

	case WM_DESTROY:
		{
			CStatusView *pStatus=GetStatusView(hwnd);

			DeleteObject(pStatus->m_hfontStatus);
			pStatus->OnDestroy();
		}
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


void CStatusView::UpdateItem(int ID)
{
	if (m_hwnd!=NULL) {
		RECT rc;

		GetItemRect(ID,&rc);
		if (rc.right>rc.left)
			InvalidateRect(m_hwnd,&rc,TRUE);
	}
}


bool CStatusView::GetItemRect(int ID,RECT *pRect) const
{
	int Index,i;
	RECT rc;

	Index=IDToIndex(ID);
	if (Index<0)
		return false;
	GetClientRect(&rc);
	for (i=0;i<Index;i++) {
		if (m_pItemList[i]->GetVisible())
			rc.left+=m_pItemList[i]->GetWidth()+STATUS_MARGIN*2;
	}
	rc.right=rc.left;
	if (m_pItemList[Index]->GetVisible())
		rc.right+=m_pItemList[i]->GetWidth()+STATUS_MARGIN*2;
	*pRect=rc;
	return true;
}


void CStatusView::GetRect(RECT *pRect) const
{
	GetWindowRect(m_hwnd,pRect);
}


int CStatusView::Height() const
{
	RECT rc;

	GetWindowRect(m_hwnd,&rc);
	return rc.bottom-rc.top;
}


int CStatusView::GetItemHeight() const
{
	RECT rc;

	GetClientRect(&rc);
	return rc.bottom-rc.top;
}


void CStatusView::SetVisible(bool fVisible)
{
	int i;

	CBasicWindow::SetVisible(fVisible);
	for (i=0;i<m_NumItems;i++)
		m_pItemList[i]->OnVisibleChange(fVisible && m_pItemList[i]->GetVisible());
}


void CStatusView::SetSingleText(LPCTSTR pszText)
{
	delete [] m_pszSingleText;
	if (pszText!=NULL) {
		m_pszSingleText=new TCHAR[lstrlen(pszText)+1];
		lstrcpy(m_pszSingleText,pszText);
		m_fSingleMode=true;
	} else {
		m_pszSingleText=NULL;
		m_fSingleMode=false;
	}
	if (m_hwnd!=NULL) {
		InvalidateRect(m_hwnd,NULL,TRUE);
		UpdateWindow(m_hwnd);
	}
}


void CStatusView::SetColor(COLORREF crBack,COLORREF crText,
							COLORREF crHighlightBack,COLORREF crHighlightText)
{
	m_crBackColor=crBack;
	m_crTextColor=crText;
	m_crHighlightBackColor=crHighlightBack;
	m_crHighlightTextColor=crHighlightText;
	if (m_hwnd!=NULL)
		InvalidateRect(m_hwnd,NULL,TRUE);
}


bool CStatusView::SetFont(HFONT hfont)
{
	DeleteObject(m_hfontStatus);
	m_hfontStatus=hfont;
	if (m_hwnd!=NULL) {
		HDC hdc;
		HFONT hfontOld;
		TEXTMETRIC tm;
		RECT rc;

		hdc=GetDC(m_hwnd);
		hfontOld=(HFONT)SelectObject(hdc,m_hfontStatus);
		GetTextMetrics(hdc,&tm);
		SelectObject(hdc,hfontOld);
		ReleaseDC(m_hwnd,hdc);
		GetClientRect(&rc);
		rc.bottom=tm.tmHeight+STATUS_MARGIN*2;
		AdjustWindowRectEx(&rc,GetWindowStyle(m_hwnd),FALSE,GetWindowExStyle(m_hwnd));
		SetWindowPos(m_hwnd,NULL,0,0,rc.right-rc.left,rc.bottom-rc.top,
			SWP_NOZORDER | SWP_NOMOVE);
		InvalidateRect(m_hwnd,NULL,TRUE);
	}
	return true;
}


int CStatusView::GetCurItem() const
{
	if (m_HotItem<0)
		return -1;
	return m_pItemList[m_HotItem]->GetID();
}


bool CStatusView::SetItemOrder(const int *pOrderList)
{
	int i,j;
	CStatusItem *pNewList[MAX_STATUS_ITEMS];

	for (i=0;i<m_NumItems;i++) {
		pNewList[i]=GetItem(IDToIndex(pOrderList[i]));
		if (pNewList[i]==NULL)
			return false;
		for (j=0;j<i;j++) {
			if (pNewList[i]==pNewList[j])
				return false;
		}
	}
	for (i=0;i<m_NumItems;i++)
		m_pItemList[i]=pNewList[i];
	if (m_hwnd!=NULL && !m_fSingleMode)
		InvalidateRect(m_hwnd,NULL,TRUE);
	return true;
}


bool CStatusView::DrawItemPreview(CStatusItem *pItem,HDC hdc,const RECT *pRect,bool fHighlight) const
{
	HBRUSH hbr;
	HFONT hfontOld;
	int OldBkMode;
	COLORREF crOldTextColor,crBkColor,crOldBkColor;

	hfontOld=(HFONT)SelectObject(hdc,m_hfontStatus);
	OldBkMode=SetBkMode(hdc,TRANSPARENT);
	crOldTextColor=SetTextColor(hdc,fHighlight?m_crHighlightTextColor:m_crTextColor);
	crBkColor=fHighlight?m_crHighlightBackColor:m_crBackColor;
	crOldBkColor=SetBkColor(hdc,crBkColor);
	hbr=CreateSolidBrush(crBkColor);
	FillRect(hdc,pRect,hbr);
	DeleteObject(hbr);
	pItem->DrawPreview(hdc,pRect);
	SetBkColor(hdc,crOldBkColor);
	SetTextColor(hdc,crOldTextColor);
	SetBkMode(hdc,OldBkMode);
	SelectObject(hdc,hfontOld);
	return true;
}
